% Symmetric Optimum Design for Vehicle Control
% Vehicle Model Parameters
La = 2E-3;
Ra = 10;
Ja = 3E-6;
cPhiF = .05;
T_R = 0;
T_R1 = 0;
tau_R = 0;
K_R = 1;
K_loop = 0;

% =======================================
% Feedback Control: Symmetric Optimum
% =======================================
% controller parameters
K = 1/cPhiF;
s1 = .5*(-Ra/La + sqrt((Ra/La)^2 - 4*cPhiF^2/La/Ja));
s2 = .5*(-Ra/La - sqrt((Ra/La)^2 - 4*cPhiF^2/La/Ja));
T1 = -1/s1;
tau = -1/s2;
T_I = 4*tau;
K_R1 = T1/2/K/4/tau^2;
K_R = K_R1*T_I;
K_loop = 1;
% Transfer function
s = tf('s');
G1 = cPhiF/Ra/(1 + La/Ra*s);
G2 = (1 + La/Ra*s)/(1 + Ra*Ja/cPhiF^2*s + La*Ja/cPhiF^2*s^2)*Ra/cPhiF^2;

% Controller transfer function
p3 = 0;
p2 = 0;
p1 = 1.45;
p0 = 1781;
l3 = 0;
l2 = 0;
l1 = 1;
l0 = 0;
% pre-filter
f0 = 1;
f1 = 0;
f2 = 0;
f3 = 0;
Rabs = 1;
% ==========================
% simulation without saturation
% ===========================
% reference tracking
r_sine = 0;
d_sine = 0;
r = 20;
M_L = 0;
sat = 100;
options = simset('MaxStep',0.0001);
sim('ECE441_Ex11_simulink',[0,.05],options);
figure(1);
hold;
plot(t,u);
grid
xlabel('time [sec]')
ylabel('Input Voltage [V]')
figure(3);
hold;
plot(t,y);
grid
xlabel('time [sec]')
ylabel('Angular velocity [rad/sec]')
% disturbance rejection
r = 0; 
M_L = 1E-3;
sim('ECE441_Ex11_simulink',[0,.05],options);
figure(2);
hold
plot(t,u);
grid
xlabel('time [sec]')
ylabel('Voltage input [V]')
figure(4);
hold
plot(t,y);
grid
xlabel('time [sec]')
ylabel('Angular velocity [rad/sec]')
% ==========================
% simulation with saturation
% ==========================
sat = 10;
% reference tracking
r = 20;
M_L = 0;
options = simset('MaxStep',0.0001);
sim('ECE441_Ex11_simulink',[0,.05],options);
figure(1);
plot(t,u,'k--');
legend('No saturation','Saturation')
figure(3);
plot(t,y,'k--');
legend('No saturation','Saturation')
% disturbance rejection
r = 0; 
M_L = 1E-3;
sim('ECE441_Ex11_simulink',[0,.05],options);
figure(2);
plot(t,u,'k--');
legend('No saturation','Saturation')
figure(4);
plot(t,y,'k--');
legend('No saturation','Saturation')

% ==========================
% simulation with extended pole placement
% ==========================
G = minreal(G1*G2);
% A(s) = s^2 + 5000 s + 4.167E5
% B(s) = 8.333E6
% R(s) = (s+2000)^4 = s^4 + 8000 s^3 + 2.4e007 s^2 + 3.2e010 s + 1.6e013
r4 = 1;
r3 = 8000;
r2 = 2.4E7;
r1 = 3.2E10;
r0 = 1.6E13;
% Controller transfer function
l3 = 0;
l2 = r4;
l1 = r3-5000;
l0 = 0;
p3 = 0;
p2 = (r2 - 5000*l1 - 4.167E5*l2)/8.333E6;
p1 = (r1 - 4.167E5*l1)/8.333E6;
p0 = r0/8.333E6;
% pre-filter
f0 = 1;
f1 = p1/p0;
f2 = p2/p0;
f3 = 0;
sat = 100;
% reference tracking
r = 20;
M_L = 0;
options = simset('MaxStep',0.0001);
sim('ECE441_Ex11_simulink',[0,.05],options);
figure(5);
plot(t,u,'k--');
grid;
xlabel('time [sec]')
ylabel('Input Voltage [V]')
figure(7);
plot(t,y,'k--');
grid;
xlabel('time [sec]')
ylabel('Angular velocity [rad/sec]')
% disturbance rejection
r = 0; 
M_L = 1E-3;
sim('ECE441_Ex11_simulink',[0,.05],options);
figure(6);
plot(t,u,'k--');
grid;
xlabel('time [sec]')
ylabel('Input Voltage [V]')
figure(8);
plot(t,y,'k--');
grid;
xlabel('time [sec]')
ylabel('Angular velocity [rad/sec]')

% ==========================
% simulation with disturbance compensation
% ==========================
% Controller transfer function
% R(s) = s^5 + 10000 s^4 + 4e007 s^3 + 8e010 s^2 + 8e013 s + 3.2e016
r4 = 10000;
r3 = 4E7;
r2 = 8E10;
r1 = 8E13;
r0 = 3.2E16;
k = (2*pi*50)^2;
ks = 2*pi*50;
b0 = 8.333E6;
a2 = 1;
a1 = 5000;
a0 = 4.167E5;
l1p = 1;
l0p = (r4 - a1*l1p)/a2;
p3 = (r3 - a2*k*l1p - a1*l0p - a0*l1p)/b0;
p2 = (r2 - a2*k*l0p - a1*k*l1p - a0*l0p)/b0;
p1 = (r1 - a1*k*l0p - a0*k*l1p)/b0;
p0 = (r0 - a0*k*l0p)/b0; 
l0 = k*l0p;
l1 = k*l1p;
l2 = l0p;
l3 = l1p;

G = 1/cPhiF/(1 + Ja*Ra/cPhiF^2*s + Ja*La/cPhiF^2*s^2);
G = cPhiF/Ja/La/(s^2 + Ra/La*s + cPhiF^2/Ja/La);

Rabs = abs((ks*i)^5 + 10000 *(ks*i)^4 + 4e007 *(ks*i)^3 + 8e010 *(ks*i)^2 + 8e013 *ks*i + 3.2e016);
C = (p3*s^3 + p2*s^2  + p1*s + p0)/(l3*s^3 + l2*s^2 + l1*s + l0);
T = 1/(1 + 1/G/C);
% pre-filter
f0 = 3.18e016;
f1 = 7.749e013;
f2 =  7.693e010;
f3 = 1.449e007;
sat = Inf;
% reference tracking
r_sine = 20;
d_sine = 0;
r = 0;
M_L = 0;
options = simset('MaxStep',0.0001);
sim('ECE441_Ex11_simulink',[0,.05],options);
figure(9);
plot(t,u,'k--');
grid;
xlabel('time [sec]')
ylabel('Input Voltage [V]')
figure(11);
plot(t,y,'k--');
grid;
xlabel('time [sec]')
ylabel('Angular velocity [rad/sec]')
% disturbance rejection
r_sine = 0;
d_sine = 1E-3;
r = 0; 
M_L = 0;
sim('ECE441_Ex11_simulink',[0,.05],options);
figure(10);
plot(t,u,'k--');
grid;
xlabel('time [sec]')
ylabel('Input Voltage [V]')
figure(12);
plot(t,y,'k--');
grid;
xlabel('time [sec]')
ylabel('Angular velocity [rad/sec]')

